package xyz.zhouxy.plusone.domain;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import xyz.zhouxy.plusone.commons.annotation.Overridable;

/**
 * 实体
 *
 * <p>
 * DDD 中的实体，带有 ID。
 * </p>
 *
 * <p>
 * 维护一个 {@link DomainEvent} 的列表，持久化时可将其中的领域事件进行发布。
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 * @see IValueObject
 * @see AggregateRoot
 */
public abstract class Entity<ID extends Serializable> {

    private final List<DomainEvent> domainEvents = new ArrayList<>();

    public abstract Optional<ID> getId();

    protected final void addDomainEvent(DomainEvent domainEvent) {
        domainEvents.add(domainEvent);
    }

    public final List<DomainEvent> getDomainEvents() {
        return List.copyOf(domainEvents);
    }

    @Override
    @Overridable
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    @Override
    @Overridable
    public int hashCode() {
        return super.hashCode();
    }

    @Override
    @Overridable
    public String toString() {
        return super.toString();
    }
}
