package xyz.zhouxy.plusone.jdbc;

import java.io.Serializable;
import java.util.Optional;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.google.common.base.Preconditions;

import xyz.zhouxy.plusone.commons.annotation.Overridable;
import xyz.zhouxy.plusone.domain.AggregateRoot;
import xyz.zhouxy.plusone.domain.IRepository;

public abstract class JdbcRepositorySupport<T extends AggregateRoot<ID>, ID extends Serializable>
        extends JdbcEntityDaoSupport<T, ID>
        implements IRepository<T, ID> {

    protected JdbcRepositorySupport(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
    }

    @Overridable
    protected abstract void doDelete(@Nonnull T entity);

    @Overridable
    protected abstract Optional<T> doFindById(@Nonnull ID id);

    @Overridable
    protected abstract T doInsert(@Nonnull T entity);

    @Overridable
    protected abstract T doUpdate(@Nonnull T entity);

    @Override
    public final void delete(T entity) {
        Preconditions.checkArgument(entity != null, "Cannot delete null.");
        doDelete(entity);
    }

    @Override
    public final Optional<T> find(ID id) {
        Preconditions.checkArgument(id != null, "Id cannot be null.");
        return doFindById(id);
    }

    @Override
    public final T save(T entity) {
        Preconditions.checkArgument(entity != null, "Cannot save null.");
        return entity.getId().isPresent() ? doUpdate(entity) : doInsert(entity);
    }

    protected abstract SqlParameterSource generateParamSource(ID id, @Nonnull T entity);

    protected final SqlParameterSource generateParamSource(@Nonnull T entity) {
        return generateParamSource(entity.getId().orElseThrow(), entity);
    }
}
