package xyz.zhouxy.plusone.system.application.web.controller;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import xyz.zhouxy.plusone.commons.util.UnifiedResponse;
import xyz.zhouxy.plusone.system.application.service.AccountContextService;
import xyz.zhouxy.plusone.system.application.service.command.ChangePasswordCommand;
import xyz.zhouxy.plusone.system.application.service.command.ChangePasswordWithoutLoginCommand;

/**
 * 账号查询本身相关信息
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@RestController
@RequestMapping("account")
public class AccountContextController {

    private final AccountContextService service;

    public AccountContextController(AccountContextService service) {
        this.service = service;
    }

    @GetMapping("info")
    public UnifiedResponse getAccountInfo() {
        adminAuthLogic.checkLogin();
        var result = service.getAccountInfo();
        return UnifiedResponse.success("查询成功", result);
    }

    @GetMapping("logout")
    public UnifiedResponse logout() {
        service.logout();
        return UnifiedResponse.success("注销成功");
    }

    @GetMapping("menus")
    public UnifiedResponse getMenuTree() {
        adminAuthLogic.checkLogin();
        var result = service.getMenuTree();
        return UnifiedResponse.success("查询成功", result);
    }

    @PostMapping("changePassword")
    public UnifiedResponse changePassword(ChangePasswordCommand command) {
        service.changePassword(command);
        return UnifiedResponse.success("修改成功，请重新登录。");
    }

    @PostMapping("changePasswordWithoutLogin")
    public UnifiedResponse changePasswordWithoutLogin(ChangePasswordWithoutLoginCommand command) {
        service.changePasswordWithoutLogin(command);
        return UnifiedResponse.success("修改成功，请重新登录。");
    }
}
