package xyz.zhouxy.plusone.system.domain.model.account;

import java.util.Objects;
import java.util.regex.Pattern;

import javax.annotation.Nullable;

import cn.hutool.core.util.DesensitizedUtil;
import xyz.zhouxy.plusone.commons.annotation.StaticFactoryMethod;
import xyz.zhouxy.plusone.commons.annotation.ValueObject;
import xyz.zhouxy.plusone.commons.constant.PatternConsts;

/**
 * 值对象：电子邮箱地址
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@ValueObject
public final class Email extends Principal {

    public static final Pattern REGEX = PatternConsts.EMAIL;

    private Email(String email) {
        super(email, REGEX);
    }

    @StaticFactoryMethod(Email.class)
    public static Email of(String email) {
        return new Email(email);
    }

    @StaticFactoryMethod(Email.class)
    @Nullable
    public static Email ofNullable(String email) {
        return Objects.nonNull(email) ? new Email(email) : null;
    }

    @Override
    public String safeValue() {
        return DesensitizedUtil.email(value());
    }
}
